/* packet-cebolla.c
 *
 * Copyright 2002 Zach Brown <zab@zabbo.net>
 * based on guidance by Mike "International Incident" Shaver
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifdef HAVE_CONFIG_H
# include "config.h"
#endif

#include <stdio.h>
#include <stdlib.h>

#ifdef HAVE_SYS_TYPES_H
# include <sys/types.h>
#endif

#ifdef HAVE_NETINET_IN_H
# include <netinet/in.h>
#endif

#ifdef NEED_SNPRINTF_H
# ifdef HAVE_STDARG_H
#  include <stdarg.h>
# else
#  include <varargs.h>
# endif
# include "snprintf.h"
#endif

#include <string.h>
#include <glib.h>
#include "packet.h"
#include "strutil.h"
#include "packet-ip.h"

#define VAL_ELEMENT(FOO) { FOO, #FOO }

#define UDP_PORT_CEBOLLA 48484

#define PROTO_STR "CEBOLLA"

static dissector_handle_t ip_handle;

/* Initialize the protocol and registered fields */
static int proto_cebolla = -1;

#define NEG_ID_BYTES 16
#define NEG_LEN (NEG_ID_BYTES*2+2+2)
/* w_neg_hdr */
static int hf_cebolla_neg_type = -1;
static int hf_cebolla_neg_error = -1;
static int hf_cebolla_neg_init_id = -1;
static int hf_cebolla_neg_resp_id = -1;

#define LNK_LEN (1+1+2+4)
/* w_lnk_hdr */
static int hf_cebolla_lnk_type = -1;
static int hf_cebolla_lnk_flags = -1;
static int hf_cebolla_lnk_seq = -1;
static int hf_cebolla_lnk_id = -1;

#define TUN_LEN (2+1+1)
/* w_lnk_hdr */
static int hf_cebolla_tun_seq = -1;
static int hf_cebolla_tun_type = -1;
static int hf_cebolla_tun_flags = -1;

/* XXX we should probably make blob types searchable someday */

/* subtree pointers */
static gint ett_cebolla = -1;
static gint ett_cebolla_neg = -1;
static gint ett_cebolla_lnk = -1;
static gint ett_cebolla_lnk_dup = -1;
static gint ett_cebolla_tun = -1;

#define NEG_INIT        0
#define NEG_REPLY       1
#define NEG_CONFREQ     2
#define NEG_CONFACK     3

static const value_string neg_type_vals[] = {
	VAL_ELEMENT(NEG_INIT),
	VAL_ELEMENT(NEG_REPLY),
	VAL_ELEMENT(NEG_CONFREQ),
	VAL_ELEMENT(NEG_CONFACK),
	{ 0, NULL }
};

#define NEG_ERR_OK			0
#define NEG_ERR_BAD_SEC			1
#define NEG_ERR_BAD_AUTH		2
#define NEG_ERR_TEMP_FAILURE		3
#define NEG_ERR_INVALID_SIGNATURE 	4
#define NEG_ERR_NO_SIGNATURE 		5
#define NEG_ERR_BAD_AUTH_LIST 		6
#define NEG_ERR_BAD_SEC_LIST		7

static const value_string neg_error_vals[] = {
	VAL_ELEMENT(NEG_ERR_OK),
	VAL_ELEMENT(NEG_ERR_BAD_SEC),
	VAL_ELEMENT(NEG_ERR_BAD_AUTH),
	VAL_ELEMENT(NEG_ERR_TEMP_FAILURE),
	VAL_ELEMENT(NEG_ERR_INVALID_SIGNATURE),
	VAL_ELEMENT(NEG_ERR_NO_SIGNATURE),
	VAL_ELEMENT(NEG_ERR_BAD_AUTH_LIST),
	VAL_ELEMENT(NEG_ERR_BAD_SEC_LIST),
	{ 0, NULL }
};

#define LINK_LSA            4
#define LINK_LSA_ACK        5
#define LINK_REKEY_REQUEST  6
#define LINK_ECHO_REQUEST   7
#define LINK_ECHO_REPLY     8
#define LINK_TUNNEL_NEG     9
#define LINK_TUNNEL_MSGEND  10
#define LINK_TUNNEL_MSGHOP  11
#define LINK_TUNNEL_DESTROY 12

static const value_string lnk_type_vals[] = {
	VAL_ELEMENT(LINK_LSA),
	VAL_ELEMENT(LINK_LSA_ACK),
	VAL_ELEMENT(LINK_REKEY_REQUEST),
	VAL_ELEMENT(LINK_ECHO_REQUEST),
	VAL_ELEMENT(LINK_ECHO_REPLY),
	VAL_ELEMENT(LINK_TUNNEL_NEG),
	VAL_ELEMENT(LINK_TUNNEL_MSGEND),
	VAL_ELEMENT(LINK_TUNNEL_MSGHOP),
	VAL_ELEMENT(LINK_TUNNEL_DESTROY),
	{ 0, NULL }
};

#define MSG_TYPE_IPV4	0

static const value_string tun_type_vals[] = {
	VAL_ELEMENT(MSG_TYPE_IPV4),
	{ 0, NULL }
};

#define B_AUTH_LIST		0
#define B_AUTH_NULL		1
#define B_AUTH_GPG		2
#define B_AUTH_SIGNATURE	3
#define B_SEC_LIST		4
#define B_SEC_DH		5

#define B_TUNNEL_ID		6

#define B_IP_ADDR		7
#define B_IP_NETMASK		8

static const value_string blob_type_vals[] = {
	VAL_ELEMENT(B_AUTH_LIST),
	VAL_ELEMENT(B_AUTH_NULL),
	VAL_ELEMENT(B_AUTH_GPG),
	VAL_ELEMENT(B_AUTH_SIGNATURE),
	VAL_ELEMENT(B_SEC_LIST),
	VAL_ELEMENT(B_SEC_DH),
	VAL_ELEMENT(B_TUNNEL_ID),
	VAL_ELEMENT(B_IP_ADDR),
	VAL_ELEMENT(B_IP_NETMASK)
};

static const gchar *
tvb_to_str(tvbuff_t *tvb, int offset, int len)
{
	const void *ptr = tvb_get_ptr(tvb, offset, len);

	return bytes_to_str(ptr, len);
}

int
dissect_cebolla_blobs(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree,
		int offset)
{
	guint16 blob_type, blob_len;
	char *blob_data = "Blob Data", 
		*blob_data_trunc = "Blob Data [Truncated!]";
	char *blob_string;

	while ( tvb_length_remaining(tvb, offset) >= 4 ) {
		
		blob_type = tvb_get_ntohs(tvb, offset);
		proto_tree_add_text(tree, tvb, offset, 2, "Blob Type: %s",
				val_to_str(blob_type, blob_type_vals, "Unknown"));
		offset += 2;


		blob_len = tvb_get_ntohs(tvb, offset);
		proto_tree_add_text(tree, tvb, offset, 2, "Blob Len: %d", blob_len);
		offset += 2;


		if ( tvb_length_remaining(tvb, offset) < blob_len )  {
			blob_len = tvb_length_remaining(tvb, offset);
			blob_string = blob_data_trunc;
		} else
			blob_string = blob_data;

		proto_tree_add_text(tree, tvb, offset, blob_len, blob_string);

		offset += blob_len;
	}

	return offset;
}

int
dissect_cebolla_neg(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree,
		int offset, char *title)
{
	proto_item *pi;
	proto_tree *neg_tree;
	guint16 type;
	guint16 error;

	pi = proto_tree_add_text(tree, tvb, offset, NEG_LEN, title);

	neg_tree = proto_item_add_subtree(pi, ett_cebolla_neg);

	type = tvb_get_guint8(tvb, offset);
	proto_tree_add_uint(neg_tree, hf_cebolla_neg_type, 
			tvb, offset, 1,
			type);
	offset += 1;

	/* pad */
	offset++;
	
	if ( check_col(pinfo->fd, COL_INFO) ) 
		col_add_str(pinfo->fd, COL_INFO,
			val_to_str(type, neg_type_vals, "Unknown type"));

	error = tvb_get_ntohs(tvb, offset);
	proto_tree_add_uint(neg_tree, hf_cebolla_neg_error, 
			tvb, offset, 2,
			error);
	offset += 2;

	proto_tree_add_bytes_format(neg_tree, hf_cebolla_neg_init_id, tvb, 
			offset, NEG_ID_BYTES, 
			tvb_get_ptr(tvb, offset, NEG_ID_BYTES), 
			"Initiator ID: %s", 
			tvb_to_str(tvb, offset, NEG_ID_BYTES));

	offset += NEG_ID_BYTES;

	proto_tree_add_bytes_format(neg_tree, hf_cebolla_neg_resp_id, tvb, 
			offset, NEG_ID_BYTES, 
			tvb_get_ptr(tvb, offset, NEG_ID_BYTES), 
			"Responder ID: %s",
			tvb_to_str(tvb, offset, NEG_ID_BYTES));

	offset += NEG_ID_BYTES;

	offset += dissect_cebolla_blobs(tvb, pinfo, neg_tree, offset);

	return offset;
}

int
dissect_cebolla_tun(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree,
		int offset)
{
	proto_item *pi;
	proto_tree *tun_tree;
	guint16 seq;
	guint8 type;
	guint8 flags;

	pi = proto_tree_add_text(tree, tvb, offset, TUN_LEN, "Tunnel");

	tun_tree = proto_item_add_subtree(pi, ett_cebolla_tun);

	seq = tvb_get_ntohs(tvb, offset);
	proto_tree_add_uint(tun_tree, hf_cebolla_tun_seq,
			tvb, offset, 2,
			seq);
	offset += 2;

	type = tvb_get_guint8(tvb, offset);
	proto_tree_add_uint(tun_tree, hf_cebolla_tun_type,
			tvb, offset, 1,
			type);
	offset += 1;

	if ( check_col(pinfo->fd, COL_INFO) )
		col_add_str(pinfo->fd, COL_INFO,
			val_to_str(type, tun_type_vals, "Unknown type"));

	flags = tvb_get_guint8(tvb, offset);
	proto_tree_add_uint(tun_tree, hf_cebolla_tun_flags,
			tvb, offset, 1,
			flags);
	offset += 1;

	if ( type == MSG_TYPE_IPV4 ) {
		tvbuff_t *next_tvb;

		next_tvb = tvb_new_subset(tvb, offset, -1, -1);

		if ( next_tvb )
			call_dissector(ip_handle, next_tvb, pinfo, tree);

		if ( check_col(pinfo->fd, COL_INFO) )
			col_append_str(pinfo->fd, COL_INFO, "  [ CEBOLLA ]");

		if ( tvb_length_remaining(tvb, offset) > 0 )
			offset += tvb_length_remaining(tvb, offset);
	}

	return offset;
}

int
dissect_cebolla_lnk(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree,
		int offset)
{
	proto_item *pi;
	proto_tree *lnk_tree;
	guint32 id;
	guint16 seq;
	guint8 type;
	guint8 flags;

	pi = proto_tree_add_text(tree, tvb, offset, LNK_LEN*2, "Link");

	lnk_tree = proto_item_add_subtree(pi, ett_cebolla_lnk);

	type = tvb_get_guint8(tvb, offset);
	proto_tree_add_uint(lnk_tree, hf_cebolla_lnk_type,
			tvb, offset, 1,
			type);
	offset += 1;

	if ( check_col(pinfo->fd, COL_INFO) )
		col_add_str(pinfo->fd, COL_INFO,
			val_to_str(type, lnk_type_vals, "Unknown type"));

	flags = tvb_get_guint8(tvb, offset);
	proto_tree_add_uint(lnk_tree, hf_cebolla_lnk_flags,
			tvb, offset, 1,
			flags);
	offset += 1;

	seq = tvb_get_ntohs(tvb, offset);
	proto_tree_add_uint(lnk_tree, hf_cebolla_lnk_seq,
			tvb, offset, 2,
			seq);
	offset += 2;

	id = tvb_get_ntohl(tvb, offset);
	proto_tree_add_uint(lnk_tree, hf_cebolla_lnk_id,
			tvb, offset, 4,
			id);
	offset += 4;

	proto_tree_add_text(lnk_tree, tvb, offset, LNK_LEN, "Duplicate Link Header");
	offset += LNK_LEN;

	/* why on earth does this return null? */
#if 0
	pi = proto_tree_add_text(lnk_tree, tvb, offset, LNK_LEN, "Link mark 2");
	lnk_tree = proto_item_add_subtree(pi, ett_cebolla_lnk_dup);

	type = tvb_get_guint8(tvb, offset);
	proto_tree_add_text(lnk_tree, tvb, offset, 1, "Type: %s (%d)", 
			val_to_str(type, lnk_type_vals, "Unknown type"), type);
	offset += 1;

	flags = tvb_get_guint8(tvb, offset);
	proto_tree_add_text(lnk_tree, tvb, offset, 1, "Flags: 0x%x", flags);
	offset += 1;

	seq = tvb_get_ntohs(tvb, offset);
	proto_tree_add_text(lnk_tree, tvb, offset, 2, "Sequence Number: %d", 
						seq);
	offset += 2;

	id = tvb_get_ntohl(tvb, offset);
	proto_tree_add_text(lnk_tree, tvb, offset, 4, "Circuit ID: 0x%x", id);
	offset += 4;
#endif

	if ( type == LINK_TUNNEL_NEG )
		offset = dissect_cebolla_neg(tvb, pinfo, tree, offset,
				"Tunnel negotiation");

	return offset;
}

void
dissect_cebolla(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
{
	proto_item *pi;
	proto_tree *cebolla_tree = NULL;
	guint16 type;
	int offset = 0;

	pinfo->current_proto = PROTO_STR;

	type = tvb_get_guint8(tvb, 0);

	if ( check_col(pinfo->fd, COL_PROTOCOL) )
		col_add_str(pinfo->fd, COL_PROTOCOL, PROTO_STR);

	if ( check_col(pinfo->fd, COL_INFO) )
		col_clear(pinfo->fd, COL_INFO);

	pi = proto_tree_add_protocol_format(tree, proto_cebolla,
			tvb, 0, tvb_length(tvb),
			"Cebolla");
	cebolla_tree = proto_item_add_subtree(pi, ett_cebolla);

	/* NEG_INIT Is 0 and gcc emits a warning :/ */
	if ( /*NEG_INIT <= type && */type <= NEG_CONFACK )
		offset = dissect_cebolla_neg(tvb, pinfo, cebolla_tree, offset,
				"Link negotiation");

	if ( LINK_LSA <= type && type <= LINK_TUNNEL_DESTROY )
		offset = dissect_cebolla_lnk(tvb, pinfo, cebolla_tree, offset);

	if ( LINK_TUNNEL_MSGEND <= type && type <= LINK_TUNNEL_MSGHOP )
		offset = dissect_cebolla_tun(tvb, pinfo, cebolla_tree, offset);

	if ( tvb_length_remaining(tvb, offset) > 0)
		proto_tree_add_text(cebolla_tree, 
				tvb, offset, tvb_length_remaining(tvb, offset),
				"Extra bytes");
}

void
proto_register_cebolla(void)
{
	static hf_register_info hf[] = {
		/* negotiation */
		{ &hf_cebolla_neg_init_id,
			{"Initiator ID", "cebolla.neg.init_id", 
				FT_BYTES, BASE_NONE, NULL, 0x0,
				 "Iniitiator ID" }},
		{ &hf_cebolla_neg_resp_id,
			{"Responder ID", "cebolla.neg.resp_id", 
				FT_BYTES, BASE_NONE, NULL, 0x0,
				 "Responder ID" }},
		{ &hf_cebolla_neg_type,
			{"Type", "cebolla.neg.type", 
				FT_UINT8, BASE_DEC, VALS(neg_type_vals), 0x0,
				 "" }},
		{ &hf_cebolla_neg_error,
			{"Error", "cebolla.neg.error", 
				FT_UINT16, BASE_DEC, VALS(neg_error_vals), 0x0,
				 "" }},

		{ &hf_cebolla_lnk_type,
			{"Type", "cebolla.lnk.type", 
				FT_UINT8, BASE_DEC, VALS(lnk_type_vals), 0x0,
				 "" }},
		{ &hf_cebolla_lnk_flags,
			{"Flags", "cebolla.lnk.flags", 
				FT_UINT8, BASE_HEX, NULL, 0x0,
				 "" }},
		{ &hf_cebolla_lnk_seq,
			{"Sequence Number", "cebolla.lnk.seq", 
				FT_UINT16, BASE_DEC, NULL, 0x0,
				 "" }},
		{ &hf_cebolla_lnk_id,
			{"Circuit ID", "cebolla.lnk.id", 
				FT_UINT8, BASE_HEX, NULL, 0x0,
				 "" }},

		{ &hf_cebolla_tun_type,
			{"Type", "cebolla.tun.type", 
				FT_UINT8, BASE_DEC, VALS(tun_type_vals), 0x0,
				 "" }},
		{ &hf_cebolla_tun_flags,
			{"Flags", "cebolla.tun.flags", 
				FT_UINT8, BASE_HEX, NULL, 0x0,
				 "" }},
		{ &hf_cebolla_tun_seq,
			{"Sequence Number", "cebolla.tun.seq", 
				FT_UINT16, BASE_DEC, NULL, 0x0,
				 "" }},
	};

	static gint *ett[] = {
		&ett_cebolla,
		&ett_cebolla_neg,
		&ett_cebolla_lnk,
		&ett_cebolla_lnk_dup,
		&ett_cebolla_tun,
	};

	proto_cebolla = proto_register_protocol("Cebolla", PROTO_STR,
							"cebolla");

	proto_register_field_array(proto_cebolla, hf, array_length(hf));
	proto_register_subtree_array(ett, array_length(ett));
}

void
proto_reg_handoff_scaw(void)
{
	ip_handle = find_dissector("ip");

	dissector_add("udp.port", UDP_PORT_CEBOLLA, 
			dissect_cebolla, proto_cebolla);
}
