/* -*- mode: c; c-basic-offset: 4; indent-tabs-mode: nil; -*-
 * vim:expandtab:shiftwidth=4:tabstop=4:
 *
 *    (c) Copyright 2002 Zach Brown <zab@zabbo.net>
 *    (c) Copyright 2001 Phil Schwan <phil@off.net>
 *
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License version 2 as
 *      published by the Free Software Foundation.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <netinet/in.h>
#include <arpa/inet.h>

#include <glib.h>

#include "main.h"

static struct command link_cmd = {
    name:       "link",
    children:   NULL,
};

CMD_FUNC(add_func)
{
    printf("buf %p\n", shm_buf);

    while(argc--) 
        printf("argc %d [%s]\n", argc, *(argv++));

    daemon_command(1);
}

CMD_FUNC(show_func)
{
    int len;
    struct cmd_link_show cls, *clsp;
    struct in_addr in;
   
    len = daemon_command(CMD_LINK_SHOW);

    for ( clsp = shm_buf; len >= sizeof(cls) ; len -= sizeof(cls), clsp++) {
        memcpy(&cls, clsp, sizeof(cls));
        memcpy(&in, &cls.addr, sizeof(cls.addr));

        printf("%08x: %s:%d: <%s,%s,%s>\n", 
                htonl(cls.id), inet_ntoa(in), ntohs(cls.port),
                (cls.flags & CMD_LINK_F_UP) ? "UP" : "DOWN",
                (cls.flags & CMD_LINK_F_NEI) ? "NEIGH" : "CLIENT",
                (cls.flags & CMD_LINK_F_INIT) ? "INIT" : "RESP");

        printf("    %10s %10s %4s %4s %10s\n"
               "    %10llu %10llu %3llu%% %4d %10llu\n", 

                "RX: pkts", "drop", "loss", "dec", "dec_bytes",

                cls.rx_packets, cls.missed_packets, 
                cls.rx_packets ? 
                    (cls.missed_packets * 100) / cls.rx_packets
                      : 0,
                cls.cs.dec,
                cls.cs.dec_bytes);

        printf("    %10s %5s %10s\n"
               "    %10llu %5d %10llu\n", 

                "TX: pkts", "enc", "eec_bytes",

                cls.tx_packets, 
                cls.cs.enc,
                cls.cs.enc_bytes);
    }
}

static struct command link_add = {
    name:       "add",
    func:       add_func,
    children:   NULL,
    help:      "add a link with an ip address and port, or something.",
    num_args:   1,
    arg_help:   " [ ip_address ] "
};

static struct command link_show = {
    name:       "show",
    func:       show_func,
    children:   NULL,
    help:      "list the current links"
};

INIT_FUNC(link_init) 
{
    link_cmd.children = g_list_append(link_cmd.children, &link_show);
    link_cmd.children = g_list_append(link_cmd.children, &link_add);

    command_register(&link_cmd);
}
