/* -*- mode: c; c-basic-offset: 4; indent-tabs-mode: nil; -*-
 * vim:expandtab:shiftwidth=4:tabstop=4:
 *
 *    (c) Copyright 2002 Zach Brown <zab@zabbo.net>
 *    (c) Copyright 2001 Phil Schwan <phil@off.net>
 *
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License version 2 as
 *      published by the Free Software Foundation.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <errno.h>

#include <glib.h>

#include "main.h"

static struct command topo_cmd = {
    name:       "topo",
    children:   NULL,
};

CMD_FUNC(show_func)
{
    int len;
    struct w_lsa_header lh;
    void *ptr, *end;
   
    len = daemon_command(CMD_TOPO_SHOW);

    if ( len < 0 ) {
        printf("topo listing failed with %s\n", strerror(-errno));
        return;
    }

    end = shm_buf + len;
    ptr = shm_buf;

    while ( ptr + sizeof(lh) <= end ) {
        struct in_addr in;
        struct w_lsa_payload lp;
        struct w_lsa_nei ln;

        memcpy(&lh, ptr, sizeof(lh));
        ptr += sizeof(lh);

        memcpy(&in, &lh.orig_ip, sizeof(in));
        printf("[ %s : %d ]\n", 
                inet_ntoa(in), ntohs(lh.orig_port));

        if ( ptr + sizeof(lp) > end )
            continue;

        memcpy(&lp, ptr, sizeof(lp));
        ptr += sizeof(lp);

        lp.num_neighbours = ntohs(lp.num_neighbours);
        lp.data_len = ntohs(lp.data_len);

        printf("  %d neighbours\n", lp.num_neighbours);

        while ( lp.num_neighbours-- && (ptr + sizeof(ln) <= end) ) {

            memcpy(&ln, ptr, sizeof(ln));
            ptr += sizeof(ln);

            memcpy(&in, &ln.nei_ip, sizeof(in));
            printf("    %s:%d\n", 
                    inet_ntoa(in), ntohs(ln.nei_port));
        }

        printf("  %d bytes of data\n", lp.data_len);
    }
}

static struct command topo_show = {
    name:       "show",
    func:       show_func,
    children:   NULL,
    help:      "display the current topology",
};

INIT_FUNC(topo_init) 
{
    topo_cmd.children = g_list_append(topo_cmd.children, &topo_show);

    command_register(&topo_cmd);
}
