/* -*- mode: c; c-basic-offset: 4; indent-tabs-mode: nil; -*-
 * vim:expandtab:shiftwidth=4:tabstop=4:
 *
 *    (c) Copyright 2002 Zach Brown <zab@zabbo.net>
 *    (c) Copyright 2001 Phil Schwan <phil@off.net>
 *
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License version 2 as
 *      published by the Free Software Foundation.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <netinet/in.h>
#include <arpa/inet.h>

#include <glib.h>

#include "main.h"

static struct command tunnel_cmd = {
    name:       "tunnel",
    children:   NULL,
};

CMD_FUNC(add_func)
{
    struct cmd_tunnel_add *cta;
    int ret, len;
    int i;

    if ( argc < 1 )  {
        printf("not enough arguments\n");
        return;
    }

    len = sizeof(struct cmd_tunnel_add) + ((argc-1) * sizeof(guint32));

    cta = alloca(len);

    if ( cta == NULL ) {
        printf("couldn't allocate space with %d additional IPs\n", argc - 1 );
        return;
    }

    cta->id = htonl(strtoul(argv[0], NULL, 16));
    cta->num_addrs = argc - 1;

    for ( i = 0 ; i < cta->num_addrs ; i++ ) {
        cta->addrs[i] = inet_addr(argv[1 + i]);
        if ( cta->addrs[i] == INADDR_NONE ) {
            printf("invalid IP: %s\n", argv[1 + i]);
            return;
        }
    }

    memcpy(shm_buf, cta, len);

    ret = daemon_command(CMD_TUNNEL_ADD);

    if ( ret < 0 )
        printf("tunnel creation failed: %s\n", strerror(-ret));
}

CMD_FUNC(del_func)
{
    int len;
    struct cmd_tunnel_del ctd;

    ctd.link_id = htonl(strtoul(argv[0], NULL, 16));
    ctd.local_id = htonl(strtoul(argv[1], NULL, 16));

    memcpy(shm_buf, &ctd, sizeof(ctd));

    len = daemon_command(CMD_TUNNEL_DEL);

    if ( len < 0 )  {
        printf("tunnel deletion failed: %s\n", strerror(-len));
        return;
    }

    printf("tunnel deleted\n");
}


CMD_FUNC(show_func)
{
    int len;
    struct cmd_tunnel_show cts, *ctsp;
    guint32 id = 0;
    int matches = 0;

    if ( argc )
        id = strtoul(argv[0], NULL, 16);

    len = daemon_command(CMD_TUNNEL_SHOW);

    for ( ctsp = shm_buf; len >= sizeof(cts) ; len -= sizeof(cts), ctsp++) {
        struct in_addr in;
        memcpy(&cts, ctsp, sizeof(cts));

        if ( id )
            if ( ntohl(cts.link_id) != id )
                continue;

        matches++;

        printf("rr %08x , lr %08x : lf %08x , rf %08x <%s%s%s>\n"
               "    link_r %08x link_f %08x ",
                ntohl(cts.remote_id_reverse),
                ntohl(cts.local_id_reverse), 
                ntohl(cts.local_id_forward), 
                ntohl(cts.remote_id_forward), 
                (cts.flags & CMD_TUNNEL_F_INIT) ? "INIT" : "RESP",
                (cts.flags & CMD_TUNNEL_F_NEG_FWD) ? "NEGFWD" : "",
                (cts.flags & CMD_TUNNEL_F_NEG_FWD_ACK) ? "NFACK" : "",
                ntohl(cts.link_id_reverse),
                ntohl(cts.link_id_forward));

        memcpy(&in, &cts.addr, sizeof(in));
        printf("addr %s ", inet_ntoa(in));
        memcpy(&in, &cts.netmask, sizeof(in));
        printf("nm %s dev %s\n", inet_ntoa(in), cts.dev);
    }

    if ( matches == 0 ) {
        printf("no tunnels found");
        if ( id ) 
            printf(" on link id %08x", id);
        printf("\n");
    }
}

static struct command tunnel_add = {
    name:       "add",
    func:       add_func,
    children:   NULL,
    help:      "add a tunnel over an existing link",
    num_args:   0,
    arg_help:   "[ hex link id ] [ 2nd hop IP ] [ 3rd hop ] [etc..]"
};

static struct command tunnel_show = {
    name:       "show",
    func:       show_func,
    children:   NULL,
    help:      "list the current tunnels",
    num_args:   0,
    arg_help:  " [ hex parent link id ]"
};

static struct command tunnel_del = {
    name:       "del",
    func:       del_func,
    children:   NULL,
    help:      "delete an existing tunnel",
    num_args:   2,
    arg_help:  " [ parent link id ] [ local id under parent link ]"
};

INIT_FUNC(tunnel_init) 
{
    tunnel_cmd.children = g_list_append(tunnel_cmd.children, &tunnel_show);
    tunnel_cmd.children = g_list_append(tunnel_cmd.children, &tunnel_add);
    tunnel_cmd.children = g_list_append(tunnel_cmd.children, &tunnel_del);

    command_register(&tunnel_cmd);
}
