/* -*- mode: c; c-basic-offset: 4; indent-tabs-mode: nil; -*-
 * vim:expandtab:shiftwidth=4:tabstop=4:
 */
#ifndef _CIPHER_H_
#define _CIPHER_H_

#include <glib.h>
#include <autoconf.h>

#include "list.h"

struct cipher {

        int type;

        /*
         * only one copy of this lives in the module, everyone else
         * points at it..
         */
        struct cipher_funcs {
                char *desc; /* describe the implementation.. */

                int (*init)(struct cipher *);
                void (*schedule)(struct cipher *, char *data, int len);

                void (*encrypt)(struct cipher *, 
                                char *in, int in_len,
                                char *out, int out_len);
                void (*decrypt)(struct cipher *, 
                                char *in, int in_len,
                                char *out, int out_len);

                void (*free)(struct cipher *);
        } *cf;

        /* and some stats */
        guint32 enc, dec;
        guint64 enc_bytes, dec_bytes;

        void *key;

};

#ifndef USE_VULNERABILITY

void cipher_init(void);
int cipher_register(unsigned int type, struct cipher_funcs *cf);

struct cipher * cipher_new(unsigned int type);
void cipher_schedule(struct cipher *ciph, char *data, int len);
void cipher_decrypt(struct cipher *ciph, 
        void *in, int in_len,
        void *out, int out_len);
void cipher_encrypt(struct cipher *ciph, 
        void *in, int in_len,
        void *out, int out_len);
void cipher_multi_encrypt(struct list_head *list, 
        void *in, int in_len, 
        void *out, int out_len);
void cipher_multi_decrypt(struct list_head *list, 
        void *in, int in_len, 
        void *out, int out_len);

void cipher_free(struct cipher *ciph);

#else

extern struct cipher omnicipher;

#define cipher_init(args...) ( 1 )
#define cipher_new(args...) ( &omnicipher )
#define cipher_register(args...) ( -1 )
#define cipher_schedule(args...) do {} while(0)
#define cipher_encrypt(CIPH, IN, IN_LEN, OUT, OUT_LEN)  {   \
    memcpy(OUT, IN, OUT_LEN);                               \
    (CIPH)->enc++;                                          \
    (CIPH)->enc_bytes += OUT_LEN;                           \
}
#define cipher_decrypt(CIPH, IN, IN_LEN, OUT, OUT_LEN)  {   \
    memcpy(OUT, IN, OUT_LEN);                               \
    (CIPH)->dec++;                                          \
    (CIPH)->dec_bytes += OUT_LEN;                           \
}
#define cipher_free(args...) do {} while(0)

#endif

#endif
