/* -*- mode: c; c-basic-offset: 4; indent-tabs-mode: nil; -*-
 * vim:expandtab:shiftwidth=4:tabstop=4:
 *
 *    (c) Copyright 2002 Zach Brown <zab@zabbo.net>
 *
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License version 2 as
 *      published by the Free Software Foundation.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <unistd.h>
#include <execinfo.h>
#include <stdlib.h>
#include <stdio.h>
#include <ctype.h>
#include <stdarg.h>

#include "log.h"

/*
 * lifted from glibc's info pages..
 */
void
log_stack_trace(void)
{
    void *array[100];
    size_t size;
    char **strings;
    size_t i;
    
    size = backtrace(array, 100);
    strings = backtrace_symbols(array, size);
    
    for (i = 0; i < size; i++) {
        l_debug("%s\n", strings[i]);
    }
    
    free(strings);
}

void
do_hexdump(void *bytes, int len, char *fmt, ...)
{
    unsigned char *buf = bytes;
    unsigned char hexes[80], printables[80], *p;
    va_list ap;
    int i;

    int h = 0;

    if ( len == 0)
        return;

    va_start(ap, fmt);
    g_logv(G_LOG_DOMAIN, G_LOG_LEVEL_DEBUG, fmt, ap);
    va_end(ap);

    if (len > 2048) {
        l_debug("asked to hexdump (%d) bytes, no thanks.\n", len);
        return;
    }

    while (len > 0) {

        h = 0;
        p = printables;
        
        for (i = 0; i < 16 && i < len; i ++) {

            /* XXX errors be damned */
            h += snprintf(hexes + h, 79 - h, "%02x ", buf[i]);

            if (isprint(buf[i]))
                    *(p++) = buf[i];
            else *(p++) = '.';

            if (i == 7) {
                h += snprintf(hexes + h, 79 - h, "- ");
                *(p++) = ' ';
            }

        }

        *p = '\0';

        l_debug(" %-50s%s\n", hexes, printables);

        len -= 16;
        buf += 16;
    }
}

void
do_bitdump(void *bytes, int len, char *fmt, ...)
{
    unsigned char *buf = bytes;
    unsigned char bits[80], *b, printables[80], *p;
    va_list ap;
    int byte;
    int bitpos;

    va_start(ap, fmt);
    g_logv(G_LOG_DOMAIN, G_LOG_LEVEL_DEBUG, fmt, ap);
    va_end(ap);

    if (len > 2048) {
        l_debug("asked to bitdump (%d) bytes, no thanks.\n", len);
        return;
    }

    /*
     * so slow.. could have a 256 string lookup table, how 
     * dorky would that be? :)
     */
    while (len > 0) {

        b = bits;
        p = printables;
        
        for (byte = 0; byte < 4 && byte < len; byte ++) {

            if(byte) 
                b += snprintf(b, 80 - (bits - b), " - ");

            for( bitpos = 0x80 ; bitpos  ; bitpos >>= 1) {
                *(b++) = buf[byte] & bitpos ? '1' : '0';
            }

            if (isprint(buf[byte]))
                    *(p++) = buf[byte];
            else *(p++) = '.';

            *b = '\0';

        }

        *p = '\0';

        l_debug(" %-50s %s\n", bits, printables);

        len -= 4;
        buf += 4;
    }
}
