#ifndef _DIGEST_H_
#define _DIGEST_H_

/*
 * this should probably be a variable that is set to be the 
 * largest digest that anyone has registered being able to
 * create.
 */
#include <openssl/evp.h>
#define MAX_DIGEST_BYTES EVP_MAX_MD_SIZE

struct digest {

        int type;

        /*
         * only one copy of this lives in the module, everyone else
         * points at it..
         */
        struct digest_funcs {
                char *desc; /* describe the implementation.. */

                int (*init)(struct digest *);

                void (*update)(struct digest *, void *data, int len);
                /*
                 * returns the number of bytes written to out, 
                 * which is generally the maximal output for the
                 * digest.
                 */
                int (*complete)(struct digest *, void *out);

                void (*free)(struct digest *);
        } *df;

        void *state;

        int max_output;
};

void digest_init(void);
int digest_register(unsigned int type, struct digest_funcs *cf);

struct digest * digest_new(unsigned int type);

/*
 * contribyte len bytes at *bytes to the hash that is currently
 * being calculated in *dig.. this may be called multiple times.
 */
void digest_update(struct digest *dig, void *bytes, int len);
/*
 * fill *out with len bytes of hash material based on the hash
 * that has been stored in *dig.. updates may be called again
 * after this, but the hash input previous to _complete will
 * be lost (obviously).
 */
void digest_complete(struct digest *dig, void *out, int len);

void digest_free(struct digest *dig);

#endif
