/* -*- mode: c; c-basic-offset: 4; indent-tabs-mode: nil; -*- */

#ifndef _LINK_H_
#define _LINK_H_

#include <time.h>

#include <glib.h>

#include "util.h"
#include "pkt.h"
#include "udp.h"
#include "main.h"
#include "topo.h"
#include "seqwind.h"
#include "keys.h"
#include "list.h"

struct link {
    guint32 id;
    struct sockaddr_in sin;
    struct socket *sk;
    struct seq seq;

    int     neighbour:1,
            client:1,
            up:1,
            neg_initiator:1;

    time_t activity;

    struct link_topo lt;

    GHashTable *local_id_hash;

    struct secret *session;
    struct rekeying rek;
    int valid_new;
    struct keys *left, *right;

    guint keepalive_id;

    guint64 rx_packets;

    GHashTable *neg_fwd_hash;

    struct list_head tun_fwd_head, tun_rev_head;
    struct list_head sk_entry;

    refcount_t __refcount;
    void (*__destroy)(struct link *link);
};

struct link * link_nei_getref(struct socket *sk, struct sockaddr_in *sin);
struct link * link_id_getref(struct socket *sk, guint32 id);

void link_teardown(struct link *link, struct link *requestor);

void link_process_neg(struct socket *sk, 
        struct packet *pkt, struct sockaddr_in *from);

void link_prepare(struct socket *sk, struct sockaddr_in *sin);
void link_up(struct link *link);
void link_rekey_action(struct link *link, int actions);
void link_send_message(struct link *link, guint8 type);
int link_init(struct daemon *dae);
void link_hash_delete(gpointer data);

#endif
