/* -*- mode: c; c-basic-offset: 4; indent-tabs-mode: nil; -*-
 * vim:expandtab:shiftwidth=4:tabstop=4:
 *
 *    (c) Copyright 2002 Zach Brown <zab@zabbo.net>
 *
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License version 2 as
 *      published by the Free Software Foundation.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <unistd.h>
#include <stdlib.h>
#include <stdio.h>
#include <stdarg.h>
#include <string.h>
#include <time.h>

#include "log.h"
#include "main.h"
#include "config.h"

static GLogLevelFlags allowed_levels = G_LOG_LEVEL_MASK & ~G_LOG_LEVEL_DEBUG;

static void 
stdio_logger(const gchar *log_domain, GLogLevelFlags log_level, 
            const gchar *message, gpointer user_data)
{
    struct daemon *dae = user_data;
    char temp[100];
    time_t timt;
    struct tm *tm;

    if ( ! (log_level & allowed_levels) )
        return;

    time(&timt);
    tm = localtime(&timt);
    if ( strftime(temp, 100, "%a %b %Y %d %T %Z: ", tm) != 0)
        fprintf(dae->log_file, "%s", temp);

    fprintf(dae->log_file, "%s", message);
    fflush(dae->log_file);
}

void
log_enable_level(GLogLevelFlags log_level)
{
    allowed_levels |= log_level;
}

int
logging_init(struct daemon *dae)
{
    dae->log_file = stderr;

	g_log_set_handler(G_LOG_DOMAIN, G_LOG_LEVEL_MASK, stdio_logger, dae);

    return 1;
}

static void
logging_shutdown(gpointer data)
{
    struct daemon *dae = data;

    if ( dae->log_file != stderr )
        fclose(dae->log_file);
}

int 
logging_init_with_config(struct daemon *dae)
{
    FILE *file;
    char *filename = dae->config->log_file;

    if ( dae->config->debug ) 
        log_enable_level(G_LOG_LEVEL_DEBUG);

    if ( filename ) {
        file = fopen(filename, "a+");
        if ( file == NULL ) {
            perror(filename);
            return 0;
        }
        if ( hook_register(&dae->shutdown_hooks, logging_shutdown, dae) ) {
            fclose(file);
            return 0;
        }

        dae->log_file = file;
    } 

    return 1;
}
