/* -*- mode: c; c-basic-offset: 4; indent-tabs-mode: nil; -*-
 * vim:expandtab:shiftwidth=4:tabstop=4:
 */

#ifndef _LOG_H_
#define _LOG_H_

#include <stdarg.h>
#include <glib.h>

#include "main.h"

/*
 * errors that we can recover gracefully from, but which the sysadmin
 * will most definitely want to see.
 */
#define l_critical(args...) g_critical( G_GNUC_PRETTY_FUNCTION "():" args)

/*
 * the most dire errors that we can't recover from.  we don't know what
 * state the universe is in after these occur, so we freak out and exit.
 */
#define l_error(args...) g_error( G_GNUC_PRETTY_FUNCTION "():" args)

/*
 * informative debugging messages that can be enabled
 */
#define l_debug(args...) g_log(G_LOG_DOMAIN, G_LOG_LEVEL_DEBUG, \
                                G_GNUC_PRETTY_FUNCTION "(): " args)
/*
 * warnings that the sysadmin will want to fix, but are not 
 * life threatening
 */
#define l_warn(args...) g_warning( G_GNUC_PRETTY_FUNCTION "(): " args)

int logging_init(struct daemon *dae);
void log_enable_level(GLogLevelFlags log_level);
int logging_init_with_config(struct daemon *dae);

#endif
