/* -*- mode: c; c-basic-offset: 4; indent-tabs-mode: nil; -*-
 * vim:expandtab:shiftwidth=4:tabstop=4:
 *
 *    (c) Copyright 2002 Zach Brown <zab@zabbo.net>
 *
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License version 2 as
 *      published by the Free Software Foundation.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <unistd.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "log.h"
#include "udp.h"
#include "pkt.h"
#include "tunnel.h"
#include "cipher.h"
#include "debug.h"

void
outgoing_link(struct packet *pkt, struct link *link, guint8 type, guint32 id)
{
    struct packet *out;
    struct w_link_hdr lh;

    if ( ! link->up )
        return;

    if ( (out = pkt_getref_alloc(0, pkt->len)) == NULL) 
        return;

    /*
     * put on our two encrypted link headers
     */

    lh.seq = htons(link->seq.tx_seq++);
    lh.type = type;
    lh.id = htonl(id);
    
    pkt_pull_head(out, sizeof(lh));

    cipher_encrypt(link->left->out, 
            &lh, sizeof(lh), 
            out->head, sizeof(lh));

    pkt_pull_head(out, sizeof(lh));

    cipher_encrypt(link->right->out, 
            &lh, sizeof(lh), 
            out->head, sizeof(lh));

#if 0 /* bring back when we encrypt/mac  */
    /*
     * encrypt from the head of pkt as we expand the tail
     * of the outgoing packet..
     */
    data = pkt_push_tail(out, to_encrypt);

    cipher_encrypt(link->left.out, 
            pkt->head, to_encrypt,
            data, to_encrypt);

    /*
     * copy the remaining clear bits of pkt..
     */
    pkt_push_head(pkt, to_encrypt);

    memcpy( pkt_push_tail(out, pkt->len), 
            pkt->head, pkt->len);

    socket_sendto(link->sk, &link->sin, out);

    /* cleanup.. */
    pkt_pull_head(pkt, to_encrypt);
#endif

    memcpy( pkt_push_tail(out, pkt->len), 
            pkt->head, pkt->len);

    socket_sendto(link->sk, &link->sin, out);

    ref_release(out);
}

/*
 * we're an endpoint initiating a msg destined to the other endpiont
 * of the tunnel
 */
void
outgoing_tunnel_msgend(struct tunnel *tunnel, struct link *link,
        struct packet *pkt, guint8 type, guint32 id)
{
    struct w_tun_hdr th;
    struct packet *out;

    th.seq = htons(tunnel->end_seq.tx_seq++);
    th.type = type;

    if ( (out = pkt_getref_alloc(sizeof(th), pkt->len)) == NULL) 
        return;

    pkt_push_tail(out, pkt->len);

    pkt_pull_head(pkt, sizeof(th));
    memcpy(pkt->head, &th, sizeof(th));

    pkt_pull_head(out, sizeof(th));

    cipher_multi_encrypt( &tunnel->multi_keys_head, 
                            pkt->head, pkt->len,
                            out->head, out->len);

    outgoing_link(out, link, LINK_TUNNEL_MSGEND, id);

    pkt_push_head(pkt, sizeof(th));

    ref_release(out);
}
