/* -*- mode: c; c-basic-offset: 4; indent-tabs-mode: nil; -*-
 * vim:expandtab:shiftwidth=4:tabstop=4:
 */
#ifndef _PKT_H_
#define _PKT_H_

#include "wire.h"
#include "util.h"

struct packet {
    /*
     * the first addressable byte of the current active packet window
     */
    void *head; 
    /*
     * the address of the byte _after_ the last addressable byte
     * in the active window
     */
    void *tail;
    /*
     * the length of the active window..
     */
    unsigned int len;
    /*
     * no one outside should have to explicitly reference any of
     * these..
     */
    int is_clone;
    refcount_t __refcount;
    void (*__destroy)(struct packet *pkt);
    char *start, *end;
    struct packet *parent;
    char data[0];
};

/*
 * managing packets
 */
struct packet * pkt_getref_alloc(unsigned int headroom, unsigned int len);

/*
 * altering the active window
 */
void * pkt_push_head(struct packet *pkt, unsigned int len);
void * pkt_pull_head(struct packet *pkt, unsigned int len);
void * pkt_push_tail(struct packet *pkt, unsigned int len);
void * pkt_pull_tail(struct packet *pkt, unsigned int len);

/*
 * hooray for blobs!
 */
void * pkt_append_blob(struct packet *pkt, guint16 type, 
                       void *data, guint16 len);
int pkt_get_head_blob(struct packet *pkt, struct w_blob *blob);


/*
 * sometimes you want to append blobs of data of unknown length:
 * _start() returns a blob that is appended to the pkt.
 * append data to the tail as usual.
 * call _end() with the blob that was returned by _start()
 */
struct w_blob * pkt_append_blob_start(struct packet *pkt, guint16 type);
void pkt_append_blob_end(struct packet *pkt, struct w_blob *blob);

struct packet * pkt_clone(struct packet *pkt);

#endif
