/* -*- mode: c; c-basic-offset: 4; indent-tabs-mode: nil; -*-
 * vim:expandtab:shiftwidth=4:tabstop=4:
 *
 *    (c) Copyright 2001 Phil Schwan <phil@off.net>
 *
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License version 2 as
 *      published by the Free Software Foundation.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <errno.h>
#include <string.h>

#define LOG_FLAGS LF_AUTH
#include "auth.h"
#include "config.h"
#include "log.h"
#include "pkt.h"
#include "plugin.h"
#include "debug.h"
#include "wire.h"

#define NULL_AUTH_KEY "this is not much of a key"
#define NULL_AUTH_SIG "jacob berkman taught me to love: %d bytes"

static int
null_append_data(struct auth_instance *inst, struct packet *pkt)
{
    char *tmp = pkt_push_tail(pkt, strlen(NULL_AUTH_KEY));
    strcpy(tmp, NULL_AUTH_KEY);

    return 0;
}

static int
null_sign(struct auth_instance *inst, struct packet *pkt,
          unsigned char *data, int len)
{
    char *tmp = g_strdup_printf(NULL_AUTH_SIG, len);
    char *p = pkt_push_tail(pkt, strlen(tmp) + 1);
    strcpy(p, tmp);
    g_free(tmp);

#if 0
    /* This is usually just too much hexdump; enable as needed. */
    hexdump(data, len, "signing %d bytes of payload:\n", len);
#else
    l_debug("signing %d bytes of payload\n", len);
#endif

    return 0;
}

static int
null_verify(struct auth_instance *inst, unsigned char *data, int len,
            struct packet *pkt)
{
    struct w_blob blob;
    char *tmp;
    int tmplen, retval;

    if (!pkt_get_head_blob(pkt, &blob) || blob.type != B_AUTH_SIGNATURE) {
        l_warn("missing or invalid auth blob\n");
        return 0;
    }

    tmp = g_strdup_printf(NULL_AUTH_SIG, len);
    tmplen = strlen(tmp) + 1;

    if (tmplen == blob.len && memcmp(tmp, pkt->head, blob.len) == 0) {
        l_debug("signatures match\n");
        retval = 1;
    } else {
        hexdump(data, len, "expecting \"%s\", found \"%s\"\n", tmp, pkt->head);
        retval = 0;
    }
    g_free(tmp);
    pkt_push_head(pkt, blob.len);

    return retval;
}

struct auth_type null_type = {
    name:        "null auth type (debug)",
    auth_id:     B_AUTH_NULL,
    start_inst:  NULL,
    stop_inst:   NULL,
    append_data: null_append_data,
    sign:        null_sign,
    verify:      null_verify,
    shutdown:    NULL
};

int
init_plugin(struct plugin_data *data, struct config *config)
{
    auth_register(&null_type);
    return 0;
}
