/* -*- mode: c; c-basic-offset: 4; indent-tabs-mode: nil; -*-
 * vim:expandtab:shiftwidth=4:tabstop=4:
 *
 *    (c) Copyright 2002 Zach Brown <zab@zabbo.net>
 *
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License version 2 as
 *      published by the Free Software Foundation.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <unistd.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include <openssl/blowfish.h>

#define LOG_FLAGS 0
#include "log.h"
#include "plugin.h"
#include "cipher.h"
#include "wire.h"
#include "debug.h"

static void 
bf_encrypt(struct cipher *ciph,
                    char *in, int in_len,
                    char *out, int out_len)
{
    BF_KEY *bfk = ciph->key;
    unsigned char ivec[8];
    int num = 0;

    memset(ivec, 0, sizeof(ivec));

    BF_cfb64_encrypt(in, out, in_len, bfk, ivec, &num, BF_ENCRYPT);
}

static void 
bf_decrypt(struct cipher *ciph,
                    char *in, int in_len,
                    char *out, int out_len)
{
    BF_KEY *bfk = ciph->key;
    unsigned char ivec[8];
    int num = 0;

    memset(ivec, 0, sizeof(ivec));

    BF_cfb64_encrypt(in, out, in_len, bfk, ivec, &num, BF_DECRYPT);
}
static void 
bf_schedule(struct cipher *ciph, char *data, int len)
{
    BF_KEY *bfk = ciph->key;

    BF_set_key(bfk, len, data);
}

static int 
bf_init(struct cipher *ciph)
{
    BF_KEY *bfk;

    if ( (bfk = g_new0(BF_KEY, 1)) == NULL)
        return -1;

    ciph->key = bfk;

    return 0;
}

static void
bf_free(struct cipher *ciph)
{
    BF_KEY *bfk = ciph->key;

    free(bfk);
}

struct cipher_funcs bf_cf = {
    desc:       "OpenSSL Blowfish CFB64",
    encrypt:    bf_encrypt,
    decrypt:    bf_decrypt,
    schedule:   bf_schedule,
    init:       bf_init,
    free:       bf_free,
};

int init_plugin(struct plugin_data *data, struct config *config)
{
    if ( cipher_register(CIPHER_BF, &bf_cf) )
        return -1;

    return 0;
}
