/* -*- mode: c; c-basic-offset: 4; indent-tabs-mode: nil; -*-
 * vim:expandtab:shiftwidth=4:tabstop=4:
 *
 *    (c) Copyright 2002 Zach Brown <zab@zabbo.net>
 *
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License version 2 as
 *      published by the Free Software Foundation.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <unistd.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include <openssl/evp.h>

#define LOG_FLAGS 0
#include "log.h"
#include "plugin.h"
#include "cipher.h"
#include "wire.h"
#include "debug.h"

static char IV[EVP_MAX_IV_LENGTH];

static void 
evp_encrypt(struct cipher *ciph,
                    char *in, int in_len,
                    char *out, int out_len)
{
    EVP_CIPHER_CTX *ctx = ciph->key;
    int outl;

    EVP_EncryptInit(ctx, EVP_CIPHER_CTX_cipher(ctx), NULL, IV);
    EVP_EncryptUpdate(ctx, out, &outl, in, in_len);
    hexdump(IV, EVP_MAX_IV_LENGTH, "iv after enc\n");
    /* XXX if (outl != out_len) ? */
}

static void 
evp_decrypt(struct cipher *ciph,
                    char *in, int in_len,
                    char *out, int out_len)
{
    EVP_CIPHER_CTX *ctx = ciph->key;
    int outl;

    EVP_DecryptUpdate(ctx, out, &outl, in, in_len);
    /* XXX if (outl != out_len) ? */
}
static void 
evp_schedule(struct cipher *ciph, char *data, int len)
{
    EVP_CIPHER *evp_type;
    EVP_CIPHER_CTX *ctx = ciph->key;

    switch(ciph->type) {
        case CIPHER_BF:
            evp_type = EVP_bf_cfb();
            /*
             * I would love to see the interpretation of the man pages
             * that gives one the ability to predict that _set_key_length
             * would segfault when this magical NULL Init() isn't called.
             */
            EVP_EncryptInit(ctx, evp_type, NULL, NULL);
            EVP_CIPHER_CTX_set_key_length(ctx, len);
            break;
        default:
            l_warn("shouldn't have been able to get here\n");
            return;
            break;
    }

    EVP_EncryptInit(ctx, evp_type, data, IV);
}

static int 
evp_init(struct cipher *ciph)
{
    /*
     * are we suppoed to just allocate this?  the evp man page
     * simply doesn't mention where _CTX comes from, of course.
     */
    EVP_CIPHER_CTX *ctx;

    switch(ciph->type) {
        case CIPHER_BF:
            break;
        default:
            l_warn("got a cipher type %d we didn't register?\n", 
                     ciph->type);
            return - 1;
            break;
    }

    if ( (ctx = g_new0(EVP_CIPHER_CTX, 1)) == NULL)
        return -1;

    ciph->key = ctx;

    return 0;
}

static void
evp_free(struct cipher *ciph)
{
    EVP_CIPHER_CTX *ctx = ciph->key;

    if ( ctx ) {
        EVP_CIPHER_CTX_cleanup(ctx);
        ciph->key = NULL;
    }
}

struct cipher_funcs evp_cf = {
    desc:       "OpenSSL EVP cipher wrapper",
    encrypt:    evp_encrypt,
    decrypt:    evp_decrypt,
    schedule:   evp_schedule,
    init:       evp_init,
    free:       evp_free,
};

/*
 * It seems that EVP doesn't have the ability to distinguish between
 * doing the key scheduling and reseting the IV at the start of new message.
 * so to do our datagram stuff we'd have to do the finish/init garbage
 * for each frame.  heh.  so screw it.  'course, I'm basing this on 
 * the man page which leaves a bit to the imagination.
 */
int init_plugin(struct plugin_data *data, struct config *config)
{
#if 0
    OpenSSL_add_all_ciphers();

    if ( cipher_register(CIPHER_BF, &evp_cf) )
        return -1;

    return 0;
#endif 
    return -1;
}
