/* -*- mode: c; c-basic-offset: 4; indent-tabs-mode: nil; -*-
 * vim:expandtab:shiftwidth=4:tabstop=4:
 *
 *    (c) Copyright 2002 Zach Brown <zab@zabbo.net>
 *
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License version 2 as
 *      published by the Free Software Foundation.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <unistd.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "log.h"
#include "seqwind.h"
#include "debug.h"

void 
seq_init(struct seq *seq)
{
    seq->tx_seq = SEQ_INIT;
    seq->expected_rx = SEQ_INIT;

    seq->right = SEQ_WINDOW_PACKETS;
    seq->left = SEQ_INIT;
    seq->window_left = 0;
    memset(seq->window, 0, sizeof(seq->window));
}

/*
 * if seq_nr < left, drop
 * if seq_nr > right, ->right = seq, shift window
 * if left <= seq_nr <= right, check bitmask
 */
int
seq_process(struct seq *seq, guint16 seq_nr)
{
    guint16 new_window_left;
    size_t to_move;

    l_debug("seq: [%u] %u %s %u %s %u\n",
        seq->window_left,
        seq->left,
        seq_nr < seq->left ? "<" : ">=", 
        seq_nr,
        seq_nr > seq->right ? ">" : "<=",
        seq->right);

    if ( seq_nr < seq->left )
        return -1;

    if ( seq_nr <= seq->right ) {

        if ( W_QUERY(seq, seq_nr) )
            return -1;

        goto ok;
    }

    /*
     * ugh, we need to shift the window.  this happens every
     * LONGBITS packets.  ->right might be an uncooperatively aligned
     * number when there is packet loss...
     */

    seq->right = seq_nr;
    seq->left = seq->right - (SEQ_WINDOW_PACKETS - 1);

    new_window_left = seq->left & ~(LONGBITS -1);

    /* see if we can skip shifting our window.. */
    if ( new_window_left == seq->window_left) 
        goto ok;

    if ( (new_window_left - seq->window_left)/8 >= sizeof(seq->window) ) {
        /* 
         * special case when our previous bit window is totally
         * useless.. the new window doesn't overlap it.
         */
        seq->window_left = new_window_left;
        memset(seq->window, 0, sizeof(seq->window));

    } else {
        /*
         * the new window overlaps the old, we need to shift..
         */
        to_move = 
            sizeof(seq->window) - ((new_window_left - seq->window_left)/8);

        memmove(seq->window,
                &seq->window[ (new_window_left - seq->window_left) / LONGBITS ],
                to_move);

        memset((char *)seq->window + to_move, 0, sizeof(seq->window) - to_move);
    }

    seq->window_left = new_window_left;

    bitdump(seq->window, sizeof(seq->window), 
            "altered window wl: %ld l: %ld r: %ld\n",
            seq->window_left, seq->left, seq->right);

ok:

    seq->missed_packets += seq_nr - seq->expected_rx;
    seq->expected_rx = seq_nr + 1;
    W_SET(seq, seq_nr);

    return 0;
}
