#ifndef _SEQWIND_H_
#define _SEQWIND_H_

#define SEQ_WINDOW_PACKETS 128  /* shrug, bigger is better, right? */

/*
 * from rfc 2402:3.4.3
 *
 * The "right" edge of the window represents the highest, validated
 * Sequence Number value received on this SA.  Packets that contain
 * Sequence Numbers lower than the "left" edge of the window are
 * rejected.
 */

/*
 * seems pretty arbitrary, but its recommended by 2402, so..
 */
#define SEQ_INIT 1
#define SEQ_REKEY_WATERMARK 0x8000

#define LONGBITS (sizeof (unsigned long) * 8)

struct seq {
    guint16 tx_seq, expected_rx;

    guint16 left, right;
    /*
     * its irritating to do portable, efficient, multi-word, bit granular, 
     * so we only memcpy the window at unsigned long boundaries, and this 
     * records our current position
     */
    guint16 window_left;

    guint64 missed_packets;
    /*
     * shifting the bitfield on unsigned long boundaries means we need
     * an extra word to cover the slack..
     */
    unsigned long window[(SEQ_WINDOW_PACKETS / LONGBITS) +1];
};


#define W_QUERY(SEQ, NR) \
    SEQ->window[ (NR - SEQ->window_left) / LONGBITS ]  & \
                    (1 << ((NR - SEQ->window_left) & (LONGBITS -1)))

#define W_SET(SEQ, NR) \
    SEQ->window[ (NR - SEQ->window_left) / LONGBITS ]  |= \
                    (1 << ((NR - SEQ->window_left) & (LONGBITS -1)))

void seq_init(struct seq *seq);
int seq_process(struct seq *seq, guint16 seq_nr);

#endif
