/* -*- mode: c; c-basic-offset: 4; indent-tabs-mode: nil; -*-
 * vim:expandtab:shiftwidth=4:tabstop=4:
 */
#ifndef _TUNNEL_H_
#define _TUNNEL_H_

#include "util.h"
#include "tun.h"
#include "seqwind.h"
#include "keys.h"
#include "list.h"

#include <glib.h>

struct tunnel {
    /*
     * an id with a value of 0 either hasn't been negotiated yet
     * or is in a tunnel on the endpoints.  the negotiator's endpoint
     * will have 0 reverse ids, and the 'server' side will have 0 
     * forward ids.  these are stored in 'host' (cpu) byte order.
     */
    guint32 local_id_forward, remote_id_forward,
            local_id_reverse, remote_id_reverse;
    /*
     * only used by endpoint tunnels
     */
    struct interface *intf;
    struct seq end_seq;

    /*
     * for now we allocate an address for us, and that address +1 for the
     * remote guy, mandating a 255*.254 netmask for the ptp net.  that
     * should change someday..
     */
    guint32 addr; 
    guint32 netmask;

    /*
     * the parent links of a tunnel.  forward is only set if there
     * is another hop in the forward direction, similar for reverse..
     */
    struct link *link_forward, *link_reverse;

    struct secret *session;
    struct list_head multi_keys_head;

    struct state {
        /*
         * if we initiated the tunnel negotiation that built up this
         * tunnel.
         */
        unsigned    neg_initiator:      1,
        /*
         * when we're an internior node in the path we need to
         * forward negotiation messages on to the next hop..
         */
                    neg_forward:        1,
                    have_fwd_init_id:   1,
                    saw_confack:        1,
                    tunneled_neg:       1;
    } state;

    /*
     * this list is only used during negotiation.  when we complete
     * an iteration of negotiation we continue to tunnel negotiation
     * through the tunnel if there are more hops in this list.
     */
    GSList *next_neighbour;
    /*
     * again, negotiation only
     */
    unsigned char forwarding_init_id[NEG_ID_BYTES];

    int lame_show_generation;

    /*
     * in addition to being in our parent link's local id hashes, we 
     * are in a pair of lists that allow the parent links to walk
     * all their tunnel offsprint without having to worry about glib's
     * hash_table_foreach exploding.
     */
    struct list_head fwd_iterator, rev_iterator;

    refcount_t __refcount;
    void (*__destroy)(struct tunnel *tun);

};

void tunnel_process_neg(struct link *link, struct packet *pkt, guint32 id);
int tunnel_create(struct link *link, int num_addrs, guint32 *addrs);
void tunnel_teardown(struct tunnel *tunnel, struct link *requestor);
struct tunnel * tunnel_id_getref(struct link *link, guint32 id);
int tunnel_init(void);
void tunnel_forward_neg(struct tunnel *tunnel, struct packet *pkt);

GHashTable *local_id_hash;

#endif
