/* -*- mode: c; c-basic-offset: 4; indent-tabs-mode: nil; -*-
 * vim:expandtab:shiftwidth=4:tabstop=4:
 *
 *    (c) Copyright 2002 Zach Brown <zab@zabbo.net>
 *    (c) Copyright 2001 Phil Schwan <phil@off.net>
 *
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License version 2 as
 *      published by the Free Software Foundation.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <unistd.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>

#include <glib.h>

#include "util.h"
#include "debug.h"
#include "random.h"
#include "log.h"
#include "wire.h"

/*
 * I wonder if these two casts will cause unalignement pain..
 */
guint u32_hash(gconstpointer u32)
{
    return *(guint32 *)u32;
}

gint u32_equal(gconstpointer a, gconstpointer b)
{
    return memcmp(a, b, sizeof(guint32)) == 0 ? TRUE : FALSE;
}

guint u16_hash(gconstpointer u16)
{
    return *(guint16 *)u16;
}

gint u16_equal(gconstpointer a, gconstpointer b)
{
    return memcmp(a, b, sizeof(guint16)) == 0 ? TRUE : FALSE;
}

/*
 * hashes a NEG_ID_BYTES byte array, it "knows" that these are
 * highly random to begin with.
 */
guint 
neghdr_hash(gconstpointer a) 
{
    guint id;

    memcpy(&id, a, sizeof(id));

    return id;
}

gboolean 
neghdr_equal(gconstpointer a, gconstpointer b)
{
    return memcmp(a, b, NEG_ID_BYTES) == 0 ? TRUE : FALSE;
}

guint sin_hash(gconstpointer sin)
{
    const struct sockaddr_in *SIN = sin;

    return SIN->sin_addr.s_addr * SIN->sin_port;
}

gint sin_equal(gconstpointer a, gconstpointer b)
{
    const struct sockaddr_in *A = a, *B = b;

    /*
     * avoiding a memcmp due to possibily unitialized padding
     * in the structs..
     */
    if ( (A->sin_addr.s_addr == B->sin_addr.s_addr) &&
         (A->sin_port == B->sin_port))
        return TRUE;

    return FALSE;
}

gint sin_compare(gconstpointer a, gconstpointer b)
{
    const struct sockaddr_in *A = a, *B = b;
    gint compare;

    if ((compare = memcmp(&A->sin_addr.s_addr, &B->sin_addr.s_addr, 
                          sizeof(A->sin_addr.s_addr))))
        return compare;

    return memcmp(&A->sin_port, &B->sin_port, sizeof(A->sin_port));
}

void sin_init(struct sockaddr_in *sin)
{
    memset(sin, 0, sizeof(*sin));
    sin->sin_family = AF_INET;
}

guint32 
get_new_id(GHashTable *hash)
{
    guint32 id;
    /*
     * lets just bet that this won't spin forever.
     */
    do {
        get_random_bytes(&id, sizeof(id));
    } while ( g_hash_table_lookup( hash, &id) != NULL);

    return id;
}

int 
hook_register(GHookList *list, GHookFunc func, gpointer data)
{
    GHook *ghook = g_hook_alloc(list);

    if ( ghook == NULL ) {
        l_critical("out of mem allocating hook\n");
        return -1;
    }

    ghook->data = data;
    ghook->func = func;

    g_hook_prepend(list, ghook);

    return 0;
}

void 
pretty_rate(unsigned int bytes, unsigned int usecs, 
        double *result, char *string)
{
    int order;
    double val;
    char *strings[] = {
        [0] = "B",
        [1] = "KB",
        [2] = "MB",
        [3] = "GB",
        [4] = "TB"};

    if ( usecs == 0 )
        usecs = 1;

    val = (((double)bytes) * 1000000.0) / usecs;

    for ( order = 0; val >= 1024.0 && order < 4; order++, val/= 1024.0 )
            ;

    *result = val;
    strcpy(string, strings[order]);
}

