/* -*- mode: c; c-basic-offset: 4; indent-tabs-mode: nil; -*-
 * vim:expandtab:shiftwidth=4:tabstop=4:
 */
#ifndef _UTIL_H_
#define _UTIL_H_

#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>

#define offsetof(TYPE, MEMB) ((size_t) &((TYPE *)0)->MEMB)
#define ARRAY_SIZE(FOO) ( sizeof(FOO) / sizeof(FOO[0]) )

#define SWAP(A, B) \
        do {                                            \
                __typeof__ (A) _happy_fun_tmp;          \
                                                        \
                _happy_fun_tmp = (A);                   \
                A = (B);                                \
                B = _happy_fun_tmp;                     \
        } while (0)

#define refcount_t int
#define REF_INIT(PTR, FUNC)    \
    do {                            \
        (PTR)->__refcount = 1;      \
        (PTR)->__destroy = FUNC;    \
    } while (0)

#define ref_release(PTR)                                                \
    do {                                                                \
        if ( (PTR)->__refcount <= 0 )                                   \
            printf("%s:%d: refcount at %d before release :(\n",         \
                    __FILE__, __LINE__, (PTR)->__refcount);             \
        if ( --(PTR)->__refcount == 0 )                                 \
            (PTR)->__destroy((PTR));                                    \
    } while (0)

#define ref_get(PTR)        \
    ({                              \
        __typeof__ (PTR) __ptr = (PTR);      \
                                    \
        (PTR)->__refcount++;        \
                                    \
        __ptr;                      \
    })

guint u16_hash(gconstpointer u16);
gint u16_equal(gconstpointer a, gconstpointer b);
guint sin_hash(gconstpointer sin);
gint sin_equal(gconstpointer a, gconstpointer b);
gint sin_compare(gconstpointer a, gconstpointer b);
guint u32_hash(gconstpointer intptr);
gint u32_equal(gconstpointer a, gconstpointer b);

guint neghdr_hash(gconstpointer a);
gboolean neghdr_equal(gconstpointer a, gconstpointer b);

void sin_init(struct sockaddr_in *sin);

guint32 get_new_id(GHashTable *hash);

int hook_register(GHookList *list, GHookFunc func, gpointer data);

void pretty_rate(unsigned int bytes, unsigned int usecs, 
        double *result, char *string);

/*
 * from the glibc info page:
 *
 * "Regardless of the namespace, you use the same functions `bind' and
 * `getsockname' to set and examine a socket's address.  These functions
 * use a phony data type, `struct sockaddr *', to accept the address.  In
 * practice, the address lives in a structure of some other data type
 * appropriate to the address format you are using, but you cast its
 * address to `struct sockaddr *' when you pass it to `bind'."
 *
 * fabulous.  so to write code that doesn't produce errors we have
 * to clutter up every instance with super-lame casts?  No Thanks.
 */

#define RECVFROM(FD, BUF, LEN, FLAGS, FROM, FROMLEN) \
    recvfrom(FD, BUF, LEN, FLAGS, (struct sockaddr *)FROM, FROMLEN)
#define SENDTO(FD, MSG, LEN, FLAGS, TO, TOLEN) \
    sendto(FD, MSG, LEN, FLAGS, (const struct sockaddr *)TO, TOLEN)
#define BIND(FD, ADDR, ADDRLEN) \
    bind(FD, (struct sockaddr *)ADDR, ADDRLEN)
#define CONNECT(FD, ADDR, ADDRLEN) \
    connect(FD, (const struct sockaddr *)ADDR, ADDRLEN)

#define TV_USECS(TV) \
    (((TV)->tv_sec*1000000L) + (TV)->tv_usec)

#endif
