/* -*- mode: c; c-basic-offset: 4; indent-tabs-mode: nil; -*-
 * vim:expandtab:shiftwidth=4:tabstop=4: 
 */

#ifndef _WIRE_H_
#define _WIRE_H_

#include <glib.h>

/*
 * this file contains all the headers that will be found on the wire..
 *
 * all multibyte values are encoded in big-endian network byte order.
 */

/*
 * ----------------------------------------------------------------------
 *  generic blobs of variable length data that are used by
 *  the overly configurable parts of the protocol.  
 * ----------------------------------------------------------------------
 */
struct w_blob {
    guint16 type;
    guint16 len; /* not including header */
}  __attribute__ ((packed));

/*
 * blobs types
 */
enum {
    B_AUTH_LIST,
    B_AUTH_NULL,
    B_AUTH_GPG,
    B_AUTH_SIGNATURE,

    B_SEC_LIST,
    B_SEC_DH,

    B_TUNNEL_ID,

    B_IP_ADDR,
    B_IP_NETMASK,

    B_NEXT_HOP,
};

/*
 * ----------------------------------------------------------------------
 *  link header
 * ----------------------------------------------------------------------
 */
/*
 * this needs to stay 64bytes or less so that it can be covered by
 * a block of most block ciphers.
 */
struct w_link_hdr {
    /*
     * this id is magically delicious.  during negotiation it advertises
     * the local_id a sender has allocated to the receiver.  during
     * real packets the sender fills in the id that the rx-er advertised
     * during nego so that the rx-er can associate the incoming packet
     * with a tunnel.
     */
    guint8 type;
    guint8 flags; /* mac?  compressed? kinda goofy.. */
    guint16 seq;
    guint32 id;
}  __attribute__ ((packed));

#define LINK_LSA            4
#define LINK_LSA_ACK        5
#define LINK_REKEY_REQUEST  6
#define LINK_ECHO_REQUEST   7
#define LINK_ECHO_REPLY     8
#define LINK_TUNNEL_NEG     9
#define LINK_TUNNEL_MSGEND  10
#define LINK_TUNNEL_MSGHOP  11
#define LINK_TUNNEL_DESTROY 12
#define LINK_DESTROY        13

/*
 * ----------------------------------------------------------------------
 *  tunnel message header
 * ----------------------------------------------------------------------
 */
struct w_tun_hdr {
    guint16 seq;
    guint8  type;
    guint8  flags;
};

enum {
    MSG_TYPE_IPV4 = 0,
    MSG_TYPE_NEG = 1,
    __MSG_TYPE_LAST
};

/*
 * ----------------------------------------------------------------------
 *  negotiation 
 * ----------------------------------------------------------------------
 */
struct w_neg_hdr {
#define NEG_ID_BYTES 16
    guint8 type;
    guint8 __pad;
    guint16 error;
    guint8 initiator_id[NEG_ID_BYTES];
    guint8 responder_id[NEG_ID_BYTES];

} __attribute__ ((packed));

/*
 * types of negotiation messages
 */
#define NEG_INIT        0
#define NEG_REPLY       1
#define NEG_CONFREQ     2
#define NEG_CONFACK     3

/*
 * neg error responses..
 */
enum {
    NEG_ERR_OK = 0,
    NEG_ERR_BAD_SEC,
    NEG_ERR_BAD_AUTH,
    NEG_ERR_TEMP_FAILURE,
    NEG_ERR_INVALID_SIGNATURE,
    NEG_ERR_NO_SIGNATURE,
    NEG_ERR_BAD_AUTH_LIST,
    NEG_ERR_BAD_SEC_LIST,
    NEG_ERR_NEXTHOP_NO_LINK,
    NEG_ERR_NEXTHOP_EEXIST,
};

/*
 * ----------------------------------------------------------------------
 *  topology flooding 
 * ----------------------------------------------------------------------
 */
struct w_lsa_header {
    guint32 orig_ip;
    guint16 orig_port;
    guint32 seq;            /* XXX again, should be bignum */
}  __attribute__ ((packed));

struct w_lsa_payload {
    guint16 num_neighbours;
    guint16 data_len;
    struct w_lsa_nei {
        guint32 nei_ip;
        guint16 nei_port;
    } __attribute__ ((packed)) neis[0];
    unsigned char data[0];
}  __attribute__ ((packed));

/*
 * ----------------------------------------------------------------------
 *  block cipher systems
 * ----------------------------------------------------------------------
 */
enum {
        CIPHER_BF = 0, /* blowfish CFB, negotiated keylength  */
        CIPHER_ENIGMA = 1, /* tee hee */
        MAX_CIPHERS, /* never hits the wire.. */
};

/*
 * ----------------------------------------------------------------------
 *  message digest systems
 * ----------------------------------------------------------------------
 */
enum {
        DIGEST_SHA1 = 0,
        DIGEST_MD5 = 1,
        MAX_DIGESTS, /* never hits the wire.. */
};
#endif
